/*
 * Header file that provide C extern function.
 *
 *   Function: local maximization.
 */

# include <stdlib.h>
# include "mex.h"
# include "math.h"
# define PI 3.1415926
# define ABS(x) ((x)>0? (x):(-(x)))
# define MAX(x, y) ((x)>(y)? (x):(y))
# define MIN(x, y) ((x)<(y)? (x):(y))

/* getting the index of matlab image, (x,y) location, (sx, sy) sizes */
int px(int x, int y, int bx, int by)    
{            
   return (x + (y-1)*bx - 1); 
}

int *int_vector(int n)
{
    int *v; 
    v = (int*) mxCalloc (n, sizeof(int));
    return v;
}

int **int_matrix(int m, int n)
{
    int **mat; 
    int i; 
    
    mat = (int**) mxCalloc(m, sizeof(int*)); 
    for (i=0; i<m; i++)
        mat[i] = int_vector(n); 
    
    return mat; 
}


void free_matrix(void **a, int nrow)
{
	int i;
	for(i = 0; i < nrow; i++)
		mxFree(a[i]);
	mxFree(a);
}

 /* variables */
int N;                      /* number of orientations */
float **fI;                 /* filtered images */
float **mI;                 /* locally maximized images */
int **sinsh, **cossh;       /* store the shift to avoid repeated sin and cosin computation */
int L, ore, sub;            /* allowed ranges of shifting in location and orientation */
int sx, sy;                 /* image height and width */
 
/* store the shift values so that we do not need to repeat the sin and cos computation */ 
void storeshift()
{
    int ind, l;
    double theta;
    
    sinsh = int_matrix(N, L+L+1);
    cossh = int_matrix(N, L+L+1);
    for (ind=0; ind<N; ind++)        
    {
        theta = PI*ind/N; 
        for (l=-L; l<=L; l++)
         {
            sinsh[ind][l+L] = (int)floor(l*sub*sin(theta)+.5); 
            cossh[ind][l+L] = (int)floor(l*sub*cos(theta)+.5); 
        }   
    }
}

/* for Gabor(x, y, orientation = ind), find local maximum in image i */
float shiftmax(int ind, int x, int y, int *rm) 
{
   float m;
   int x1, y1, l, ml, mx, my, de, d, d1, here, mo, md;
 
   m = 0.;

   for (l=-L; l<=L; l++) 
   {
        x1 = x + cossh[ind][l+L];
        y1 = y + sinsh[ind][l+L];   /* shifting in normal direction */
        if ((x1>=1)&&(x1<=sx)&&(y1>=1)&&(y1<=sy))
        {
        here = px(x1, y1, sx, sy);
        for (de=-ore; de<=ore; de++)
        {
              d = de+ind;
              d1 = d;
              if (d<0)
                 d1 = d+N;
              if (d>=N)
                  d1 = d-N;   /* shifting in orientation */
              if (fI[d1][here]>m)
               {
                   m = fI[d1][here];   /* local maximization */
                   ml = l; mx = x1; my = y1; mo = d1; md = de; 
                }
             }
        }
     }
   rm[0] = ml; rm[1] = mx; rm[2] = my; rm[3] = mo; rm[4] = md; 
   return(m); 
}



/* 
 * N : # orientations
 * fI: S1 map
 * mI: M1 map
 */
void cmax(int height, int width, float **input_fI, float **input_mI, int Lrange, int Orange, int input_sub, int nOri) 
{
    int o, here;
    int x, y, rm[5];
    float tmp;
    
    N = nOri;
    
    fI = input_fI;
    mI = input_mI;
    
    sx = height;
    sy = width;
    
    L = Lrange;
    ore = Orange;
    sub = input_sub;
    
    storeshift();
 
    for (o=0; o<N; o++)
    {
       for (y=Lrange*sub+1; y<sy-Lrange*sub-1; y++)
           for (x=Lrange*sub+1; x<sx-Lrange*sub-1; x++)
           {
               here = px(x, y, sx, sy);
               tmp = shiftmax(o, x, y, rm);
               mI[o][here] = tmp;
           }
    }
}



                    