function scoreVec = calcLLRScan(model,loc,M1)

% <loc> is a structure that contains:
%   (1) <offset> (2) <sx>, <sy> (3) subSampleStep (4) N
% <offset> is the position of the subtemplate inside the whole template
% <sx>, <sy> is the template size
% <M1> is the map of R(r|b), the edge confidences
% <N> is the total number of negative patches

N = loc.N;

if ~isempty(model)

    % The following code is for scanning a PART template (i.e. a level-2
    % template). Otherwise, <M1> may not be a simple R(r|b) map.

    sx = loc.sx; sy = loc.sy; % template size
    offset = loc.offset; sub = loc.subSampleStep;
    nF = model.Totalsketch; % number of features
    Mx = model.Mx; My = model.My; Mo = model.Mi; % starts from 1
    Mm1 = model.Mm1; Mm = model.Mm;

    scoreVec = cell(nF,1);
    for iF = 1:nF
        scoreVec{iF} = zeros(N,1,'single');
    end
    count = 1;
    for i = 1:size(M1,1) % number of examples
        imsize = size(M1{i,1});
        for startX = offset(1)+(0:sub:imsize(1)-sx) % starts from 0 !
            for startY = offset(2)+(0:sub:imsize(2)-sy)
                for iF = 1:nF
                    % disp(sprintf('x=%d,y=%d,o=%d',Mx(iF)+startX,My(iF)+startY,Mo(iF)));
                    r = M1{i,Mo(iF)}(Mx(iF)+startX,My(iF)+startY);
                    r = sig(r,model.Upperbound);
                    scoreVec{iF}(count) = Mm(iF)*r - Mm1(iF); % the score is not accumulative
                                    % individual scores for each feature
                end
                count = count + 1;
            end
        end
    end
else

end


function x = sig(x,Upperbound)
x = Upperbound*(2./(1+exp(-2*x/Upperbound))-1);
