function dumpFeature_GaborS1(featureVersion,overwrite,rescale,windowSize,imagePath,allfilter,maxImArea,localHalfx,...
    Lrange, Orange, sub, Upperbound)

% compute feature maps for a single image
norient = length(allfilter);

% parameters used for UINT8 storage
RANGE = [0 6];

% check if the file already exists
if exist( ...
        sprintf('%s/%s.mat','Feature/GaborS1',...
        imagePath((1+length('Image/'):end))),'file')
    if ~(overwrite > 0)
        return;
    end
end

im = imread(imagePath); % load image file
if size(im,3)==3 % color image
    im = rgb2gray(im); % convert to gray image
end


% resize image
if rescale > 0
    sx = size(im,1); sy = size(im,2);
    if sx >= sy
        newSx = windowSize;
        newSy = floor(.5+windowSize*sy/sx);
    else
        newSy = windowSize;
        newSx = floor(.5+windowSize*sx/sy);
    end
    im = imresize(im,[newSx,newSy],'nearest');
else
    if size(im,1) * size(im,2) > maxImArea
        im = imresize(im, sqrt(maxImArea/size(im,1)/size(im,2)),'nearest');
    end
    [newSx newSy] = size(im);
end

% compute S1 maps
im = single(im); % convert to floating point (single precision) numbers
im = im - mean(im(:)); % global image normalization
im = im / std(im(:));
S1 = applyfilterfftsame({im},allfilter); % raw S1 Map (no sigmoid transformation)


localHalfy = localHalfx;
S1LocalMean = cell(1,norient);
localAveFilter = ones(localHalfx,localHalfy,'single')/single(localHalfx)/single(localHalfy);
for o = 1:norient
    S1LocalMean{o} = filter2(localAveFilter,S1{o},'same');
end

% do not use the non-saturated local mean response
S1LocalMeanRaw = cell(size(S1LocalMean)); % saturated but not locally normalized
for ii = 1:numel(S1LocalMeanRaw)
    S1LocalMeanRaw{ii} = S1LocalMean{ii} + 0;
end

S1LocalMeanOverOrientation = zeros(newSx,newSy);
for o = 1:norient
    S1LocalMeanOverOrientation = S1LocalMeanOverOrientation + 1/norient * S1LocalMean{o};
end

% local normalization for sketch feature
S1LocalNorm = cell(1,norient);
for o = 1:norient
    S1LocalNorm{o} = S1{o} ./ S1LocalMeanOverOrientation;
end

% sigmoid transformation on S1LocalNorm 
Ctransform(1, norient, S1LocalNorm,...
    newSx,...
    newSy, Upperbound);

% local normalization for sketch feature, per orientation
S1LocalNormPerOri = cell(1,norient);
for o = 1:norient
    S1LocalNormPerOri{o} = S1{o} ./ S1LocalMean{o};
end

% sigmoid transformation on S1LocalNormPerOri
Ctransform(1, norient, S1LocalNormPerOri,...
    newSx,...
    newSy, Upperbound);


% sigmoid transformation on S1
Ctransform(1, norient, S1,...
    newSx,...
    newSy, Upperbound);

% local mean on saturated Gabor responses
S1LocalMean = cell(1,norient);
localAveFilter = ones(localHalfx,localHalfy,'single')/single(localHalfx)/single(localHalfy);
for o = 1:norient
    S1LocalMean{o} = filter2(localAveFilter,S1{o},'same');
end

S1LocalMeanOverOrientation = zeros(newSx,newSy);
for o = 1:norient
    S1LocalMeanOverOrientation = S1LocalMeanOverOrientation + 1/norient * S1LocalMean{o};
end

% local normalization for texture feature (i.e. local mean feature)
for o = 1:norient
    S1LocalMean{o} = S1LocalMean{o} ./ S1LocalMeanOverOrientation;
end

disp(imagePath);
GaborSize = single(size(allfilter{1},1));

% obtain MAX1 maps
subsampleM1 = 1;
Lrange_ratio = 6.0 / 17.0;
[M1 ARGMAX1 M1RowShift M1ColShift M1OriShifted] = ...
	mexc_ComputeMAX1( norient, S1, Lrange_ratio,...
		Orange, GaborSize, subsampleM1 );

[M1LocalNorm ARGMAX1LocalNorm] = ...
	mexc_ComputeMAX1( norient, S1LocalNorm, Lrange_ratio,...
		Orange, GaborSize, subsampleM1 );

[M1LocalNormPerOri ARGMAX1LocalNormPerOri] = ...
	mexc_ComputeMAX1( norient, S1LocalNormPerOri, Lrange_ratio,...
		Orange, GaborSize, subsampleM1 );


% store all the maps in uint8 with a linear scaling
S1 = convert2uint8( S1, RANGE );
S1LocalNorm = convert2uint8( S1LocalNorm, RANGE );
S1LocalMean = convert2uint8( S1LocalMean, RANGE );
S1LocalNormPerOri = convert2uint8( S1LocalNormPerOri, RANGE );
S1LocalMeanRaw = convert2uint8( S1LocalMeanRaw, RANGE );
S1LocalMeanOverOrientation = convert2uint8( S1LocalMeanOverOrientation, RANGE );
M1 = convert2uint8( M1, RANGE );
M1LocalNorm = convert2uint8( M1LocalNorm, RANGE );
M1LocalNormPerOri = convert2uint8( M1LocalNormPerOri, RANGE );

save(sprintf('%s/%s/%s.mat','Feature/',featureVersion,imagePath((1+length('Image/'):end))),... % image and filter parameters
	'GaborSize','maxImArea','windowSize',... % image and filter parameters
    'S1','S1LocalNorm','S1LocalMean',...
    'S1LocalNormPerOri','localHalfx','S1LocalMeanRaw','S1LocalMeanOverOrientation',...
	'RANGE',... % for storing in uint8
	'M1', 'M1LocalNorm', 'M1LocalNormPerOri',... % for MAX1 maps
	'ARGMAX1', 'ARGMAX1LocalNorm', 'ARGMAX1LocalNormPerOri',... % for MAX1 maps
	'M1RowShift', 'M1ColShift', 'M1OriShifted'); % for MAX1 maps

%%
