function [ap testAUC tp fp rec prec EER featureNoRange] = evaluate(tssPos,tssNeg,mode)
% <mode> = 0 : the scores are individual feature scores
% <mode> = 1 : the scores are accumulated feature scores
% This is for scanning window testing only.

% by default, the scores are individual scores
if nargin < 3
    mode = 0;
end

M = length(tssPos{1});
N = length(tssNeg{1});
maxNF = length(tssPos);
ap = zeros(maxNF,1);
testAUC = zeros(maxNF,1);
tp = cell(maxNF,1); fp = cell(maxNF,1);
rec = cell(maxNF,1); prec = cell(maxNF,1);
EER = zeros(maxNF,1);
accumulatedPosScore = zeros(M,1,'single');
accumulatedNegScore = zeros(N,1,'single');

featureSubStep = 1; % the sub-sample rate for #feature

for nF = 1:maxNF
    % Draw ROC curve and calculate AUC
    if mode == 0
        accumulatedNegScore(:) = tssNeg{nF}(:) + accumulatedNegScore(:);
        accumulatedPosScore(:) = tssPos{nF}(:) + accumulatedPosScore(:);
        allScore = [accumulatedPosScore;accumulatedNegScore]; % scores of all test images
    else
        % mode == 1
        % the score is already accumulated
        allScore = [tssPos{nF}(:);tssNeg{nF}(:)]; % scores of all test images
    end
    gt = [ones(M,1);-1*ones(N,1)]; % ground truth of all test images
    [testAUC(nF) tp{nF}, fp{nF}] = calc_auc(allScore,gt); % obtain ROC curve by comparing predicted scores and ground truth
     % and true positive and false postive rate
    [ap(nF) rec{nF} prec{nF} EER(nF)] = getAP(gt,allScore);
end

featureNoRange = featureSubStep:featureSubStep:maxNF;
ap = single(ap(featureNoRange));
testAUC = single(testAUC(featureNoRange));
tp = (tp(featureNoRange));
fp = (fp(featureNoRange));
rec = (rec(featureNoRange));
prec = (prec(featureNoRange));
EER = single(EER(featureNoRange));
